@@ -39,11 +39,11 @@
 
 #include "gdm-settings-desktop-backend.h"
 
-#define GDM_SETTINGS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_SETTINGS, GdmSettingsPrivate))
-
-struct GdmSettingsPrivate
+struct _GdmSettings
 {
-        GdmSettingsBackend *backend;
+        GObject   parent;
+
+        GList    *backends;
 };
 
 enum {
@@ -80,15 +80,28 @@ gdm_settings_get_value (GdmSettings *settings,
 {
         GError  *local_error;
         gboolean res;
+        GList   *l;
 
         g_return_val_if_fail (GDM_IS_SETTINGS (settings), FALSE);
         g_return_val_if_fail (key != NULL, FALSE);
 
         local_error = NULL;
-        res = gdm_settings_backend_get_value (settings->priv->backend,
-                                              key,
-                                              value,
-                                              &local_error);
+
+        for (l = settings->backends; l; l = g_list_next (l)) {
+                GdmSettingsBackend *backend = l->data;
+
+                if (local_error) {
+                        g_error_free (local_error);
+                        local_error = NULL;
+                }
+
+                res = gdm_settings_backend_get_value (backend,
+                                                      key,
+                                                      value,
+                                                      &local_error);
+                if (res)
+                        break;
+        }
         if (! res) {
                 g_propagate_error (error, local_error);
         }
@@ -104,6 +117,7 @@ gdm_settings_set_value (GdmSettings *settings,
 {
         GError  *local_error;
         gboolean res;
+        GList   *l;
 
         g_return_val_if_fail (GDM_IS_SETTINGS (settings), FALSE);
         g_return_val_if_fail (key != NULL, FALSE);
@@ -111,10 +125,23 @@ gdm_settings_set_value (GdmSettings *settings,
         g_debug ("Setting value %s", key);
 
         local_error = NULL;
-        res = gdm_settings_backend_set_value (settings->priv->backend,
-                                              key,
-                                              value,
-                                              &local_error);
+
+        for (l = settings->backends; l; l = g_list_next (l)) {
+                GdmSettingsBackend *backend = l->data;
+
+                if (local_error) {
+                        g_error_free (local_error);
+                        local_error = NULL;
+                }
+
+                res = gdm_settings_backend_set_value (backend,
+                                                      key,
+                                                      value,
+                                                      &local_error);
+                if (res)
+                        break;
+        }
+
         if (! res) {
                 g_propagate_error (error, local_error);
         }
@@ -133,7 +160,7 @@ gdm_settings_class_init (GdmSettingsClass *klass)
                 g_signal_new ("value-changed",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmSettingsClass, value_changed),
+                              0,
                               NULL,
                               NULL,
                               g_cclosure_marshal_generic,
@@ -142,8 +169,6 @@ gdm_settings_class_init (GdmSettingsClass *klass)
                               G_TYPE_STRING,
                               G_TYPE_STRING,
                               G_TYPE_STRING);
-
-        g_type_class_add_private (klass, sizeof (GdmSettingsPrivate));
 }
 
 static void
@@ -162,13 +187,25 @@ backend_value_changed (GdmSettingsBackend *backend,
 static void
 gdm_settings_init (GdmSettings *settings)
 {
-        settings->priv = GDM_SETTINGS_GET_PRIVATE (settings);
+        GList *l;
+        GdmSettingsBackend *backend;
 
-        settings->priv->backend = gdm_settings_desktop_backend_new ();
-        g_signal_connect (settings->priv->backend,
-                          "value-changed",
-                          G_CALLBACK (backend_value_changed),
-                          settings);
+        backend = gdm_settings_desktop_backend_new (GDM_CUSTOM_CONF);
+        if (backend)
+                settings->backends = g_list_prepend (NULL, backend);
+
+        backend = gdm_settings_desktop_backend_new (GDM_RUNTIME_CONF);
+        if (backend)
+                settings->backends = g_list_prepend (settings->backends, backend);
+
+        for (l = settings->backends; l; l = g_list_next (l)) {
+                backend = l->data;
+
+                g_signal_connect (backend,
+                                  "value-changed",
+                                  G_CALLBACK (backend_value_changed),
+                                  settings);
+        }
 }
 
 static void
@@ -181,11 +218,13 @@ gdm_settings_finalize (GObject *object)
 
         settings = GDM_SETTINGS (object);
 
-        g_return_if_fail (settings->priv != NULL);
+        g_return_if_fail (settings != NULL);
 
-        if (settings->priv->backend != NULL) {
-                g_object_unref (settings->priv->backend);
-        }
+        g_list_foreach (settings->backends, (GFunc) g_object_unref, NULL);
+        g_list_free (settings->backends);
+        settings->backends = NULL;
+
+        settings_object = NULL;
 
         G_OBJECT_CLASS (gdm_settings_parent_class)->finalize (object);
 }
