@@ -64,9 +64,8 @@ struct _GdmLocalDisplayFactory
         guint            seat_removed_id;
 
 #if defined(ENABLE_WAYLAND_SUPPORT) && defined(ENABLE_USER_DISPLAY_SERVER)
-        char            *tty_of_active_vt;
+        unsigned int     active_vt;
         guint            active_vt_watch_id;
-        guint            wait_to_finish_timeout_id;
 #endif
 };
 
@@ -270,20 +269,18 @@ gdm_local_display_factory_create_transient_display (GdmLocalDisplayFactory *fact
         return ret;
 }
 
-static gboolean
+static void
 finish_display_on_seat_if_waiting (GdmDisplayStore *display_store,
                                    GdmDisplay      *display,
                                    const char      *seat_id)
 {
         if (gdm_display_get_status (display) != GDM_DISPLAY_WAITING_TO_FINISH)
-                return FALSE;
+                return;
 
         g_debug ("GdmLocalDisplayFactory: finish background display\n");
         gdm_display_stop_greeter_session (display);
         gdm_display_unmanage (display);
         gdm_display_finish (display);
-
-        return FALSE;
 }
 
 static void
@@ -300,6 +297,25 @@ finish_waiting_displays_on_seat (GdmLocalDisplayFactory *factory,
                                    seat_id);
 }
 
+static void
+on_session_registered_cb (GObject *gobject,
+                          GParamSpec *pspec,
+                          gpointer user_data)
+{
+        GdmDisplay *display = GDM_DISPLAY (gobject);
+        GdmLocalDisplayFactory *factory = GDM_LOCAL_DISPLAY_FACTORY (user_data);
+        gboolean registered;
+
+        g_object_get (display, "session-registered", &registered, NULL);
+
+        if (!registered)
+                return;
+
+        g_debug ("GdmLocalDisplayFactory: session registered on display, looking for any background displays to kill");
+
+        finish_waiting_displays_on_seat (factory, "seat0");
+}
+
 static void
 on_display_status_changed (GdmDisplay             *display,
                            GParamSpec             *arg1,
@@ -378,6 +394,13 @@ on_display_status_changed (GdmDisplay             *display,
         case GDM_DISPLAY_PREPARED:
                 break;
         case GDM_DISPLAY_MANAGED:
+#if defined(ENABLE_WAYLAND_SUPPORT) && defined(ENABLE_USER_DISPLAY_SERVER)
+                g_signal_connect_object (display,
+                                         "notify::session-registered",
+                                         G_CALLBACK (on_session_registered_cb),
+                                         factory,
+                                         0);
+#endif
                 break;
         case GDM_DISPLAY_WAITING_TO_FINISH:
                 break;
@@ -606,14 +629,6 @@ lookup_by_session_id (const char *id,
 }
 
 #if defined(ENABLE_WAYLAND_SUPPORT) && defined(ENABLE_USER_DISPLAY_SERVER)
-static gboolean
-wait_to_finish_timeout (GdmLocalDisplayFactory *factory)
-{
-        finish_waiting_displays_on_seat (factory, "seat0");
-        factory->wait_to_finish_timeout_id = 0;
-        return G_SOURCE_REMOVE;
-}
-
 static void
 maybe_stop_greeter_in_background (GdmLocalDisplayFactory *factory,
                                   GdmDisplay             *display)
@@ -645,16 +660,8 @@ maybe_stop_greeter_in_background (GdmLocalDisplayFactory *factory,
         }
 
         g_debug ("GdmLocalDisplayFactory: killing login window once its unused");
+
         g_object_set (G_OBJECT (display), "status", GDM_DISPLAY_WAITING_TO_FINISH, NULL);
-
-        /* We stop the greeter after a timeout to avoid flicker */
-        if (factory->wait_to_finish_timeout_id != 0)
-                g_source_remove (factory->wait_to_finish_timeout_id);
-
-        factory->wait_to_finish_timeout_id =
-                g_timeout_add_seconds (WAIT_TO_FINISH_TIMEOUT,
-                                       (GSourceFunc)wait_to_finish_timeout,
-                                       factory);
 }
 
 static gboolean
@@ -663,13 +670,12 @@ on_vt_changed (GIOChannel    *source,
                GdmLocalDisplayFactory *factory)
 {
         GIOStatus status;
-        static const char *tty_of_initial_vt = "tty" GDM_INITIAL_VT;
-        g_autofree char *tty_of_previous_vt = NULL;
         g_autofree char *tty_of_active_vt = NULL;
         g_autofree char *login_session_id = NULL;
         g_autofree char *active_session_id = NULL;
+        unsigned int previous_vt, new_vt;
         const char *session_type = NULL;
-        int ret;
+        int ret, n_returned;
 
         g_debug ("GdmLocalDisplayFactory: received VT change event");
         g_io_channel_seek_position (source, 0, G_SEEK_SET, NULL);
@@ -705,38 +711,43 @@ on_vt_changed (GIOChannel    *source,
 
         g_strchomp (tty_of_active_vt);
 
+        errno = 0;
+        n_returned = sscanf (tty_of_active_vt, "tty%u", &new_vt);
+
+        if (n_returned != 1 || errno != 0) {
+                g_critical ("GdmLocalDisplayFactory: Couldn't read active VT (got '%s')",
+                            tty_of_active_vt);
+                return G_SOURCE_CONTINUE;
+        }
+
         /* don't do anything if we're on the same VT we were before */
-        if (g_strcmp0 (tty_of_active_vt, factory->tty_of_active_vt) == 0) {
+        if (new_vt == factory->active_vt) {
                 g_debug ("GdmLocalDisplayFactory: VT changed to the same VT, ignoring");
                 return G_SOURCE_CONTINUE;
         }
 
-        tty_of_previous_vt = g_steal_pointer (&factory->tty_of_active_vt);
-        factory->tty_of_active_vt = g_steal_pointer (&tty_of_active_vt);
+        previous_vt = factory->active_vt;
+        factory->active_vt = new_vt;
 
         /* don't do anything at start up */
-        if (tty_of_previous_vt == NULL) {
-                g_debug ("GdmLocalDisplayFactory: VT is %s at startup",
-                         factory->tty_of_active_vt);
+        if (previous_vt == 0) {
+                g_debug ("GdmLocalDisplayFactory: VT is %u at startup",
+                         factory->active_vt);
                 return G_SOURCE_CONTINUE;
         }
 
-        g_debug ("GdmLocalDisplayFactory: VT changed from %s to %s",
-                 tty_of_previous_vt, factory->tty_of_active_vt);
+        g_debug ("GdmLocalDisplayFactory: VT changed from %u to %u",
+                 previous_vt, factory->active_vt);
 
         /* if the old VT was running a wayland login screen kill it
          */
         if (gdm_get_login_window_session_id ("seat0", &login_session_id)) {
-                unsigned int vt;
+                unsigned int login_window_vt;
 
-                ret = sd_session_get_vt (login_session_id, &vt);
-                if (ret == 0 && vt != 0) {
-                        g_autofree char *tty_of_login_window_vt = NULL;
-
-                        tty_of_login_window_vt = g_strdup_printf ("tty%u", vt);
-
-                        g_debug ("GdmLocalDisplayFactory: tty of login window is %s", tty_of_login_window_vt);
-                        if (g_strcmp0 (tty_of_login_window_vt, tty_of_previous_vt) == 0) {
+                ret = sd_session_get_vt (login_session_id, &login_window_vt);
+                if (ret == 0 && login_window_vt != 0) {
+                        g_debug ("GdmLocalDisplayFactory: VT of login window is %u", login_window_vt);
+                        if (login_window_vt == previous_vt) {
                                 GdmDisplayStore *store;
                                 GdmDisplay *display;
 
@@ -746,7 +757,6 @@ on_vt_changed (GIOChannel    *source,
                                 display = gdm_display_store_find (store,
                                                                   lookup_by_session_id,
                                                                   (gpointer) login_session_id);
-
                                 if (display != NULL)
                                         maybe_stop_greeter_in_background (factory, display);
                         } else {
@@ -759,7 +769,7 @@ on_vt_changed (GIOChannel    *source,
          * on it (unless a login screen is already running elsewhere, then
          * jump to that login screen)
          */
-        if (strcmp (factory->tty_of_active_vt, tty_of_initial_vt) != 0) {
+        if (factory->active_vt != GDM_INITIAL_VT) {
                 g_debug ("GdmLocalDisplayFactory: active VT is not initial VT, so ignoring");
                 return G_SOURCE_CONTINUE;
         }
@@ -829,16 +839,10 @@ gdm_local_display_factory_stop_monitor (GdmLocalDisplayFactory *factory)
                 factory->seat_removed_id = 0;
         }
 #if defined(ENABLE_WAYLAND_SUPPORT) && defined(ENABLE_USER_DISPLAY_SERVER)
-        if (factory->wait_to_finish_timeout_id != 0) {
-                g_source_remove (factory->wait_to_finish_timeout_id);
-                factory->wait_to_finish_timeout_id = 0;
-        }
         if (factory->active_vt_watch_id) {
                 g_source_remove (factory->active_vt_watch_id);
                 factory->active_vt_watch_id = 0;
         }
-
-        g_clear_pointer (&factory->tty_of_active_vt, g_free);
 #endif
 }
 
